/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.threeD;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.*;

final class JBorderChooser extends JDialog {
	int 				type 		= -1;
	int				init			= -1;
	boolean		SELECTED	= false;
	JToggleButton	[] toggle	= new JToggleButton[ThreeDUtils.BORDER_TYPES];
	JButton		ok			= new JButton("OK");
	JButton		reset		= new JButton("Reset");
	JButton		cancel		= new JButton("Cancel");
	
	public JBorderChooser()  {
		setModal(true);
		
		JPanel contentPane = (JPanel)getContentPane();
		contentPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
		contentPane.add(createBorderPanel());
		contentPane.add(new Strut());
		contentPane.add(new JPopupMenu.Separator());
		contentPane.add(new Strut());
		contentPane.add(createButtons());
		
		pack();
		Dimension size = getSize();
		if (size.width < 300) size.width = 300;
		if (size.height < 300) size.height = 300;
		setSize(size);
	}

	private void cancel()  {
		setVisible(false);
	}

	private JPanel createBorderPanel()  {
		JPanel result = new JPanel(new GridLayout(0, ThreeDUtils.BORDER_TYPES/2, 4, 4));
		
		ButtonGroup grp = new ButtonGroup();
		for (int i=0; i<ThreeDUtils.BORDER_TYPES; ++i)
			{
			toggle[i] = makeToggleButton(i);
			result.add(toggle[i]);
			grp.add(toggle[i]);
			}
		
		return result;
	}

	private Box createButtons()  {
		Box result = Box.createHorizontalBox();
		
		result.add(ok);
		result.add(new Strut());
		result.add(Box.createHorizontalGlue());
		result.add(reset);
		result.add(Box.createHorizontalGlue());
		result.add(new Strut());
		result.add(cancel);
		
		ok.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { ok(); }});
		reset.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { reset(); }});
		cancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { cancel(); }});
		
		return result;
	}

	private JToggleButton makeToggleButton(final int t)  {
		JToggleButton result = new JToggleButton();
		
		result.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) { type = t;	 }
		});
		
		result.setIcon(new Icon() {
			private int w=100, h=50;
			public int getIconWidth() { return w; }
			public int getIconHeight() { return h; }
			public void paintIcon(Component c, Graphics g, int x, int y) { ThreeDUtils.drawBorder(t, g, x, y, w, h); }
		});
		
		return result;
	}

	private void ok()  {
		SELECTED = true;
		setVisible(false);
	}

	private void reset()  {
		toggle[init].setSelected(true);
	}

	public int showDialog(JComponent parent, String title, int defaultType)  {
		setTitle(title);
		
		type = defaultType;
		init = type;
		reset();
		
		SELECTED = false;
		setLocation(parent.getLocationOnScreen());
		setVisible(true);
		
		return SELECTED ? type : -1;
	}

}
