/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.threeD;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.plaf.*;

class ThreeDIcons {
	public static final int RIGHT 	= 0;
	public static final int LEFT 	= 1;
	public static final int UP 		= 2;
	public static final int DOWN	= 3;
	
	final static class ArrowIcon implements Icon, UIResource {
		private int 		orientation 	= RIGHT;
		private int 		size			= 15;
		private Color	color		= Color.black;
		
		public ArrowIcon()  {
		}

		public ArrowIcon(int orientation, int size)  {
			this.orientation = orientation;
			this.size = size;
		}

		public int getIconHeight()  {
			if (orientation == LEFT || orientation == RIGHT)
				{
				return size * 2 - 1;
				}
			else 
				{
				return size;
				}
		}

		public int getIconWidth()  {
			if (orientation == LEFT || orientation == RIGHT)
				{
				return size;
				}
			else 
				{
				return size * 2 - 1;
				}
		}

		public void paintIcon(Component c, Graphics g, int x, int y)  {
			Point p1 = null;
			Point p2 = null;
			Point p3 = null;
			
			switch (orientation)
				{
				case LEFT:
					p1 = new Point(0, size-1);
					p2 = new Point(size-1, 0);
					p3 = new Point(size-1, size*2-2);
					break;
				case RIGHT:
					p1 = new Point(0,0);
					p2 = new Point(0, size*2-2);
					p3 = new Point(size-1, size-1);
					break;
				case UP:
					p1 = new Point(size-1, 0);
					p2 = new Point(0, size-1);
					p3 = new Point(size*2-2, size-1);
					break;
				case DOWN:
				default:
					p1 = new Point(0,0);
					p2 = new Point(size*2-2,0);
					p3 = new Point(size-1, size-1);
					break;
				}
			
			Polygon p = new Polygon();
			p.addPoint(p1.x,p1.y);
			p.addPoint(p2.x,p2.y);
			p.addPoint(p3.x,p3.y);
			
			g.setColor(color);
			g.translate(x,y);
			g.fillPolygon(p);
			g.translate(-x,-y);
		}

		public void setColor(Color c)  {
			color = c;
		}

		public void setOrientation(int newOrientation)  {
			orientation = newOrientation;
		}

		public void setSize(int newSize)  {
			size = newSize;
		}

	}

	final static class CheckBoxIcon implements Icon, UIResource {
		private static final int SIZE = 13; 
		
		public int getIconHeight()  {
			Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.checkBoxBorderType1);
			return SIZE+i.top+i.bottom;
		}

		public int getIconWidth()  {
			Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.checkBoxBorderType1);
			return SIZE+i.left+i.right;
		}

		public void paintIcon(Component c, Graphics g, int x, int y)  {
			JCheckBox cb = (JCheckBox)c;
			ButtonModel model = cb.getModel();
			
			if (model.isPressed())
				{
				g.setColor(ThreeDPrefs.highlightColor);
				g.fillRect(x, y, getIconWidth()-1, getIconHeight()-1);
				ThreeDUtils.drawBorder(ThreeDPrefs.checkBoxBorderType2, g, x, y, getIconWidth(), getIconHeight());
				}
			else
				{
				ThreeDUtils.drawBorder(ThreeDPrefs.checkBoxBorderType1, g, x, y, getIconWidth(), getIconHeight());
				}
			
			if (model.isSelected()) 
				{
				Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.checkBoxBorderType1);
				x+=i.left;
				y+=i.top;
				g.setColor(ThreeDPrefs.textColor);
				g.fillRect( x+3, y+5, 2, SIZE-8 );
				g.drawLine( x+(SIZE-4), y+3, x+5, y+(SIZE-6) );
				g.drawLine( x+(SIZE-4), y+4, x+5, y+(SIZE-5) );
				}
		}

	}

	final static class CheckBoxMenuIcon implements Icon, UIResource {
		private static final int SIZE = 11;
		
		public int getIconHeight()  {
			Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.checkBoxBorderType1);
			return SIZE+i.top+i.bottom;
		}

		public int getIconWidth()  {
			Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.checkBoxBorderType1);
			return SIZE+i.left+i.right;
		}

		public void paintIcon(Component c, Graphics g, int x, int y)  {
			JCheckBoxMenuItem cb = (JCheckBoxMenuItem)c;
			ButtonModel model = cb.getModel();
			
			if (model.isPressed())
				{
				g.setColor(ThreeDPrefs.highlightColor);
				g.fillRect(x, y, getIconWidth()-1, getIconHeight()-1);
				ThreeDUtils.drawBorder(ThreeDPrefs.checkBoxBorderType2, g, x, y, getIconWidth(), getIconHeight());
				}
			else
				{
				ThreeDUtils.drawBorder(ThreeDPrefs.checkBoxBorderType1, g, x, y, getIconWidth(), getIconHeight());
				}
			
			if (model.isSelected()) 
				{
				Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.checkBoxBorderType1);
				x+=i.left;
				y+=i.top;
				g.setColor(ThreeDPrefs.textColor);
				g.fillRect( x+3, y+4, 2, SIZE-7 );
				g.drawLine( x+(SIZE-4), y+3, x+5, y+(SIZE-6) );
				g.drawLine( x+(SIZE-4), y+4, x+5, y+(SIZE-5) );
				}
		}

	}

	final static class InternalFrameIcon1 implements Icon, UIResource {
		final int SIZE = 9;
		
		public int getIconHeight()  {
			return SIZE+4;
		}

		public int getIconWidth()  {
			return SIZE+4;
		}

		public void paintIcon(Component c, Graphics g, int x, int y)  {
			g.translate(x+2,y+2);
			
			g.setColor(ThreeDPrefs.textColor);
			g.drawLine(0,0,SIZE-1,SIZE-1);
			g.drawLine(SIZE-1,0,0,SIZE-1);
			
			g.translate(-x-2,-y-2);
		}

	}

	final static class InternalFrameIcon2 implements Icon, UIResource {
		final int SIZE = 9;
		
		public int getIconHeight()  {
			return SIZE+4;
		}

		public int getIconWidth()  {
			return SIZE+4;
		}

		public void paintIcon(Component c, Graphics g, int x, int y)  {
			g.translate(x+2,y+2);
			
			g.setColor(ThreeDPrefs.textColor);
			g.drawLine(SIZE/2-1,SIZE/2,SIZE/2+1,SIZE/2);
			g.drawLine(SIZE/2,SIZE/2-1,SIZE/2,SIZE/2+1);
			
			g.translate(-x-2,-y-2);
		}

	}

	final static class InternalFrameIcon3 implements Icon, UIResource {
		final int SIZE = 9;
		
		public int getIconHeight()  {
			return SIZE+4;
		}

		public int getIconWidth()  {
			return SIZE+4;
		}

		public void paintIcon(Component c, Graphics g, int x, int y)  {
			g.translate(x+2,y+2);
			
			g.setColor(ThreeDPrefs.textColor);
			g.drawRect(0,0,SIZE-1,SIZE-1);
			g.drawRect(0,SIZE/2,SIZE/2,SIZE/2);
			
			g.setColor(ThreeDPrefs.lightEdgeColor);
			g.fillRect(1,SIZE/2+1,SIZE/2-2,SIZE/2-2);
			
			g.translate(-x-2,-y-2);
		}

	}

	final static class RadioButtonIcon implements Icon, UIResource {
		static final int SIZE = 13;
		
		public int getIconHeight()  {
			Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.radioButtonBorderType1);
			return SIZE+i.top+i.bottom;
		}

		public int getIconWidth()  {
			Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.radioButtonBorderType1);
			return SIZE+i.left+i.right;
		}

		public void paintIcon(Component c, Graphics g, int x, int y)  {
			JRadioButton rb = (JRadioButton)c;
			ButtonModel model = rb.getModel();
			
			if (model.isPressed() || model.isSelected())
				{
				g.setColor(ThreeDPrefs.highlightColor);
				g.fillRect(x, y, getIconWidth()-1, getIconHeight()-1);
				ThreeDUtils.drawBorder(ThreeDPrefs.radioButtonBorderType2, g, x, y, getIconWidth(), getIconHeight());
				}
			else
				{
				ThreeDUtils.drawBorder(ThreeDPrefs.radioButtonBorderType1, g, x, y, getIconWidth(), getIconHeight());
				}
		}

	}

	final static class RadioButtonMenuIcon implements Icon, UIResource {
		static final int SIZE = 11;
		
		public int getIconHeight()  {
			Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.radioButtonBorderType1);
			return SIZE+i.top+i.bottom;
		}

		public int getIconWidth()  {
			Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.radioButtonBorderType1);
			return SIZE+i.left+i.right;
		}

		public void paintIcon(Component c, Graphics g, int x, int y)  {
			JRadioButtonMenuItem rb = (JRadioButtonMenuItem)c;
			ButtonModel model = rb.getModel();
			
			if (model.isPressed() || model.isSelected())
				{
				g.setColor(ThreeDPrefs.highlightColor);
				g.fillRect(x, y, getIconWidth()-1, getIconHeight()-1);
				ThreeDUtils.drawBorder(ThreeDPrefs.radioButtonBorderType2, g, x, y, getIconWidth(), getIconHeight());
				}
			else
				{
				ThreeDUtils.drawBorder(ThreeDPrefs.radioButtonBorderType1, g, x, y, getIconWidth(), getIconHeight());
				}
		}

	}

}
