/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.threeD;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import javax.swing.plaf.metal.*;
import javax.swing.text.*;

public class ThreeDLookAndFeel extends BasicLookAndFeel {
	
	static {
		// The next line is for the font bug :-)
		GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
	}

	private Object createTextEditorKeyBindings()  {
		return new UIDefaults.LazyInputMap(new Object[] {
			"ctrl C", DefaultEditorKit.copyAction,
			"ctrl V", DefaultEditorKit.pasteAction,
			"ctrl X", DefaultEditorKit.cutAction,
			"COPY", DefaultEditorKit.copyAction,
			"PASTE", DefaultEditorKit.pasteAction,
			"CUT", DefaultEditorKit.cutAction,
			"shift LEFT", DefaultEditorKit.selectionBackwardAction,
			"shift KP_LEFT", DefaultEditorKit.selectionBackwardAction,
			"shift RIGHT", DefaultEditorKit.selectionForwardAction,
			"shift KP_RIGHT", DefaultEditorKit.selectionForwardAction,
			"ctrl LEFT", DefaultEditorKit.previousWordAction,
			"ctrl KP_LEFT", DefaultEditorKit.previousWordAction,
			"ctrl RIGHT", DefaultEditorKit.nextWordAction,
			"ctrl KP_RIGHT", DefaultEditorKit.nextWordAction,
			"ctrl shift LEFT", DefaultEditorKit.selectionPreviousWordAction,
			"ctrl shift KP_LEFT", DefaultEditorKit.selectionPreviousWordAction,
			"ctrl shift RIGHT", DefaultEditorKit.selectionNextWordAction,
			"ctrl shift KP_RIGHT", DefaultEditorKit.selectionNextWordAction,
			"ctrl A", DefaultEditorKit.selectAllAction,
			"HOME", DefaultEditorKit.beginLineAction,
			"END", DefaultEditorKit.endLineAction,
			"shift HOME", DefaultEditorKit.selectionBeginLineAction,
			"shift END", DefaultEditorKit.selectionEndLineAction,
			"UP", DefaultEditorKit.upAction,
			"KP_UP", DefaultEditorKit.upAction,
			"DOWN", DefaultEditorKit.downAction,
			"KP_DOWN", DefaultEditorKit.downAction,
			"PAGE_UP", DefaultEditorKit.pageUpAction,
			"PAGE_DOWN", DefaultEditorKit.pageDownAction,
			"shift PAGE_UP", "selection-page-up",
			"shift PAGE_DOWN", "selection-page-down",
			"ctrl shift PAGE_UP", "selection-page-left",
			"ctrl shift PAGE_DOWN", "selection-page-right",
			"shift UP", DefaultEditorKit.selectionUpAction,
			"shift KP_UP", DefaultEditorKit.selectionUpAction,
			"shift DOWN", DefaultEditorKit.selectionDownAction,
			"shift KP_DOWN", DefaultEditorKit.selectionDownAction,
			"ENTER", DefaultEditorKit.insertBreakAction,
			"typed \010", DefaultEditorKit.deletePrevCharAction,
			"DELETE", DefaultEditorKit.deleteNextCharAction,
			"RIGHT", DefaultEditorKit.forwardAction,
			"LEFT", DefaultEditorKit.backwardAction, 
			"KP_RIGHT", DefaultEditorKit.forwardAction,
			"KP_LEFT", DefaultEditorKit.backwardAction,
			"TAB", DefaultEditorKit.insertTabAction,
			"ctrl BACK_SLASH", "unselect"/*DefaultEditorKit.unselectAction*/,
			"ctrl HOME", DefaultEditorKit.beginAction,
			"ctrl END", DefaultEditorKit.endAction,
			"ctrl shift HOME", DefaultEditorKit.selectionBeginAction,
			"ctrl shift END", DefaultEditorKit.selectionEndAction,
			"ctrl T", "next-link-action",
			"ctrl shift T", "previous-link-action",
			"ctrl SPACE", "activate-link-action",
			"control shift O", "toggle-componentOrientation"/*DefaultEditorKit.toggleComponentOrientation*/
		});
	}

	private Object createTextFieldKeyBindings()  {
		return new UIDefaults.LazyInputMap(new Object[] {
			"ctrl C", DefaultEditorKit.copyAction,
			"ctrl V", DefaultEditorKit.pasteAction,
			"ctrl X", DefaultEditorKit.cutAction,
			"COPY", DefaultEditorKit.copyAction,
			"PASTE", DefaultEditorKit.pasteAction,
			"CUT", DefaultEditorKit.cutAction,
			"shift LEFT", DefaultEditorKit.selectionBackwardAction,
			"shift KP_LEFT", DefaultEditorKit.selectionBackwardAction,
			"shift RIGHT", DefaultEditorKit.selectionForwardAction,
			"shift KP_RIGHT", DefaultEditorKit.selectionForwardAction,
			"ctrl LEFT", DefaultEditorKit.previousWordAction,
			"ctrl KP_LEFT", DefaultEditorKit.previousWordAction,
			"ctrl RIGHT", DefaultEditorKit.nextWordAction,
			"ctrl KP_RIGHT", DefaultEditorKit.nextWordAction,
			"ctrl shift LEFT", DefaultEditorKit.selectionPreviousWordAction,
			"ctrl shift KP_LEFT", DefaultEditorKit.selectionPreviousWordAction,
			"ctrl shift RIGHT", DefaultEditorKit.selectionNextWordAction,
			"ctrl shift KP_RIGHT", DefaultEditorKit.selectionNextWordAction,
			"ctrl A", DefaultEditorKit.selectAllAction,
			"HOME", DefaultEditorKit.beginLineAction,
			"END", DefaultEditorKit.endLineAction,
			"shift HOME", DefaultEditorKit.selectionBeginLineAction,
			"shift END", DefaultEditorKit.selectionEndLineAction,
			"typed \010", DefaultEditorKit.deletePrevCharAction,
			"DELETE", DefaultEditorKit.deleteNextCharAction,
			"RIGHT", DefaultEditorKit.forwardAction,
			"LEFT", DefaultEditorKit.backwardAction,
			"KP_RIGHT", DefaultEditorKit.forwardAction,
			"KP_LEFT", DefaultEditorKit.backwardAction,
			"ENTER", JTextField.notifyAction,
			"ctrl BACK_SLASH", "unselect"/*DefaultEditorKit.unselectAction*/,
			"control shift O", "toggle-componentOrientation"/*DefaultEditorKit.toggleComponentOrientation*/
		});
	}

	public UIDefaults getDefaults()  {
		UIDefaults table = super.getDefaults();
		
		return table;
	}

	public String getDescription()  {
		return "3D Look & Feel v2.7 - \u00a9 1999-2000 by Markus Hillenbrand (http://www.markus-hillenbrand.de)";
	}

	public String getID()  {
		return "3D";
	}

	public String getName()  {
		return "3D";
	}

	/**
	* Initialize the uiClassID to BasicComponentUI mapping.
	* The JComponent classes define their own uiClassID constants
	* (see AbstractComponent.getUIClassID).  This table must
	* map those constants to a BasicComponentUI class of the
	* appropriate type.
	* 
	* @see BasicLookAndFeel#getDefaults
	* 
	*/

	protected void initClassDefaults(UIDefaults table)  {
		super.initClassDefaults(table);
		
		Object[] uiDefaults = {
			"ButtonUI"					, "swing.addon.plaf.threeD.ThreeD_ButtonUI",
			"CheckBoxMenuItemUI"		, "swing.addon.plaf.threeD.ThreeD_CheckBoxMenuItemUI",
			"CheckBoxUI"				, "swing.addon.plaf.threeD.ThreeD_CheckBoxUI",
			"ColorChooserUI"				, "swing.addon.plaf.threeD.ThreeD_ColorChooserUI",
			"ColorChooserUI"				, "swing.addon.plaf.threeD.ThreeD_ColorChooserUI",
			"ComboBoxUI"				, "swing.addon.plaf.threeD.ThreeD_ComboBoxUI",
			"DesktopIconUI"				, "swing.addon.plaf.threeD.ThreeD_DesktopIconUI",
			"DesktopPaneUI"				, "swing.addon.plaf.threeD.ThreeD_DesktopPaneUI",
			"EditorPaneUI"				, "swing.addon.plaf.threeD.ThreeD_EditorPaneUI",
			"FileChooserUI"				, "swing.addon.plaf.threeD.ThreeD_FileChooserUI",
			"InternalFrameUI"				, "swing.addon.plaf.threeD.ThreeD_InternalFrameUI",
			"LabelUI"					, "swing.addon.plaf.threeD.ThreeD_LabelUI",
			"ListUI"						, "swing.addon.plaf.threeD.ThreeD_ListUI",
			"MenuBarUI"					, "swing.addon.plaf.threeD.ThreeD_MenuBarUI",
			"MenuItemUI"				, "swing.addon.plaf.threeD.ThreeD_MenuItemUI",
			"MenuUI"					, "swing.addon.plaf.threeD.ThreeD_MenuUI",
			"OptionPaneUI"				, "swing.addon.plaf.threeD.ThreeD_OptionPaneUI",
			"PanelUI"					, "swing.addon.plaf.threeD.ThreeD_PanelUI",
			"PasswordFieldUI"			, "swing.addon.plaf.threeD.ThreeD_PasswordFieldUI",
			"PopupMenuSeparatorUI"		, "swing.addon.plaf.threeD.ThreeD_PopupMenuSeparatorUI",
			"PopupMenuUI"				, "swing.addon.plaf.threeD.ThreeD_PopupMenuUI",
			"ProgressBarUI"				, "swing.addon.plaf.threeD.ThreeD_ProgressBarUI",
			"RadioButtonMenuItemUI"		, "swing.addon.plaf.threeD.ThreeD_RadioButtonMenuItemUI",
			"RadioButtonUI"				, "swing.addon.plaf.threeD.ThreeD_RadioButtonUI",
			"ScrollBarUI"					, "swing.addon.plaf.threeD.ThreeD_ScrollBarUI",
			"ScrollPaneUI"				, "swing.addon.plaf.threeD.ThreeD_ScrollPaneUI",
			"SeparatorUI"				, "swing.addon.plaf.threeD.ThreeD_SeparatorUI",
			"SliderUI"					, "swing.addon.plaf.threeD.ThreeD_SliderUI",
			"SplitPaneUI"					, "swing.addon.plaf.threeD.ThreeD_SplitPaneUI",
			"TabbedPaneUI"				, "swing.addon.plaf.threeD.ThreeD_TabbedPaneUI",
			"TableHeaderUI"				, "swing.addon.plaf.threeD.ThreeD_TableHeaderUI",
			"TableUI"					, "swing.addon.plaf.threeD.ThreeD_TableUI",
			"TextAreaUI"					, "swing.addon.plaf.threeD.ThreeD_TextAreaUI",
			"TextFieldUI"					, "swing.addon.plaf.threeD.ThreeD_TextFieldUI",
			"TextPaneUI"					, "swing.addon.plaf.threeD.ThreeD_TextPaneUI",
			"ToggleButtonUI"				, "swing.addon.plaf.threeD.ThreeD_ToggleButtonUI",
			"ToolBarSeparatorUI"			, "swing.addon.plaf.threeD.ThreeD_ToolBarSeparatorUI",
			"ToolBarUI"					, "swing.addon.plaf.threeD.ThreeD_ToolBarUI",
			"ToolTipUI"					, "swing.addon.plaf.threeD.ThreeD_ToolTipUI",
			"TreeUI"						, "swing.addon.plaf.threeD.ThreeD_TreeUI",
			"ViewportUI"					, "swing.addon.plaf.threeD.ThreeD_ViewportUI",
		};
		
		table.putDefaults(uiDefaults);
	}

	protected void initComponentDefaults(UIDefaults table)  {
		super.initComponentDefaults(table);
		
		Object [] defaults = {
			// Button
			"Button.background"			, ThreeDPrefs.backgroundColor,
			"Button.border"				, ThreeDPrefs.buttonBorder,
			"Button.font"					, ThreeDPrefs.normalFont,
			"Button.foreground"			, ThreeDPrefs.textColor,
			"Button.margin"				, new InsetsUIResource(3,5,3,5),
			"Button.textIconGap"			, new Integer(4),
			"Button.textShiftOffset"			, new Integer(0),
			// CheckBox
			"CheckBox.background"		, ThreeDPrefs.backgroundColor,
			"CheckBox.border"				, BorderFactory.createEmptyBorder(1,1,1,1),
			"CheckBox.font"				, ThreeDPrefs.normalFont,
			"CheckBox.foreground"			, ThreeDPrefs.textColor,
			"CheckBox.icon"				, ThreeDPrefs.checkBoxIcon,
			"CheckBox.margin"				, new InsetsUIResource(3,5,3,5),
			"CheckBox.textIconGap"		, new Integer(4),
			"CheckBox.textShiftOffset"		, new Integer(0),
			// CheckBoxMenuItem
			"CheckBoxMenuItem.acceleratorFont"					, ThreeDPrefs.acceleratorFont,
			"CheckBoxMenuItem.acceleratorForeground"			, ThreeDPrefs.acceleratorColor,
			"CheckBoxMenuItem.acceleratorSelectionForeground"	, ThreeDPrefs.acceleratorColor,
			"CheckBoxMenuItem.arrowIcon"						, null,
			"CheckBoxMenuItem.background"						, ThreeDPrefs.menuBackgroundColor,
			"CheckBoxMenuItem.border"							, ThreeDPrefs.menuItemBorder,
			"CheckBoxMenuItem.borderPainted"					, Boolean.TRUE,
			"CheckBoxMenuItem.checkIcon"						, new ThreeDIcons.CheckBoxMenuIcon(),
			"CheckBoxMenuItem.disabledForeground"				, ThreeDPrefs.disabledMenuTextColor,
			"CheckBoxMenuItem.font"							, ThreeDPrefs.normalFont,
			"CheckBoxMenuItem.foreground"						, ThreeDPrefs.menuTextColor,
			"CheckBoxMenuItem.margin"							, new InsetsUIResource(3,5,3,5),
			"CheckBoxMenuItem.selectionBackground"			, ThreeDPrefs.highlightColor,
			"CheckBoxMenuItem.selectionForeground"				, ThreeDPrefs.menuTextColor,
			// ColorChooser
			"ColorChooser.background"						, ThreeDPrefs.backgroundColor,
			"ColorChooser.font"								, ThreeDPrefs.normalFont,
			"ColorChooser.foreground"						, ThreeDPrefs.textColor,
			"ColorChooser.rgbBlueMnemonic"					, new Integer(KeyEvent.VK_B),
			"ColorChooser.rgbGreenMnemonic"				, new Integer(KeyEvent.VK_G),
			"ColorChooser.rgbRedMnemonic"					, new Integer(KeyEvent.VK_R),
			"ColorChooser.swatchesDefaultRecentColor"		, ThreeDPrefs.backgroundColor,
			"ColorChooser.swatchesRecentSwatchSize"			, new Dimension(10, 10),
			"ColorChooser.swatchesSwatchSize"				, new Dimension(10, 10),
			// ComboBox
			"ComboBox.background"					, ThreeDPrefs.backgroundColor,
			"ComboBox.disabledBackground"			, ThreeDPrefs.backgroundColor,
			"ComboBox.disabledForeground"			, ThreeDPrefs.disabledTextColor,
			"ComboBox.font"							, ThreeDPrefs.normalFont,
			"ComboBox.foreground"					, ThreeDPrefs.textColor,
			"ComboBox.selectionBackground"			, ThreeDPrefs.highlightColor,
			"ComboBox.selectionForeground"			, ThreeDPrefs.textColor,
			// Desktop
			"Desktop.background"					, ThreeDPrefs.desktopColor,
			// DesktopIcon
			"DesktopIcon.border"						, ThreeDPrefs.desktopIconBorder,
			"DesktopIcon.icon"						, LookAndFeel.makeIcon(getClass(), "icons/window.gif"),
			// EditorPane
			"EditorPane.background"					, ThreeDPrefs.textFieldBackgroundColor,
			"EditorPane.border"						, ThreeDPrefs.textComponentBorder,
			"EditorPane.caretBlinkRate"				, new Integer(ThreeDPrefs.caretBlinkRate),
			"EditorPane.caretForeground"				, ThreeDPrefs.caretColor,
			"EditorPane.font"							, ThreeDPrefs.normalFont,
			"EditorPane.foreground"					, ThreeDPrefs.textColor,
			"EditorPane.inactiveForeground"			, ThreeDPrefs.textColor,
			"EditorPane.focusInputMap"				, createTextEditorKeyBindings(),
			"EditorPane.margin"						, new InsetsUIResource(2,2,2,2),
			"EditorPane.selectionBackground"			, ThreeDPrefs.highlightColor,
			"EditorPane.selectionForeground"			, ThreeDPrefs.textColor,
			// FileChooser
			"FileChooser.cancelButtonMnemonic"		, new Integer(KeyEvent.VK_C),
			"FileChooser.detailsViewIcon"				, LookAndFeel.makeIcon(getClass(), "icons/DetailsView.gif"),
			"FileChooser.helpButtonMnemonic"		, new Integer(KeyEvent.VK_H),
			"FileChooser.homeFolderIcon"			, LookAndFeel.makeIcon(getClass(), "icons/HomeFolder.gif"),
			"FileChooser.listViewIcon"				, LookAndFeel.makeIcon(getClass(), "icons/ListView.gif"),
			"FileChooser.newFolderIcon"				, LookAndFeel.makeIcon(getClass(), "icons/NewFolder.gif"),
			"FileChooser.openButtonMnemonic"		, new Integer(KeyEvent.VK_O),
			"FileChooser.saveButtonMnemonic"		, new Integer(KeyEvent.VK_S),
			"FileChooser.updateButtonMnemonic"		, new Integer(KeyEvent.VK_U),
			"FileChooser.upFolderIcon"				, LookAndFeel.makeIcon(getClass(), "icons/UpFolder.gif"),
			// FileView
			"FileView.computerIcon"					, LookAndFeel.makeIcon(getClass(), "icons/Computer.gif"),
			"FileView.directoryIcon"					, LookAndFeel.makeIcon(getClass(), "icons/Directory.gif"),
			"FileView.fileIcon"						, LookAndFeel.makeIcon(getClass(), "icons/File.gif"),
			"FileView.floppyDriveIcon"					, LookAndFeel.makeIcon(getClass(), "icons/FloppyDrive.gif"),
			"FileView.hardDriveIcon"					, LookAndFeel.makeIcon(getClass(), "icons/HardDrive.gif"),
			// InternalFrame
			"InternalFrame.activeTitleBackground"		, ThreeDPrefs.highlightColor,
			"InternalFrame.activeTitleForeground"		, ThreeDPrefs.textColor,
			"InternalFrame.border"					, ThreeDPrefs.internalFrameBorder,
			"InternalFrame.closeIcon"					, LookAndFeel.makeIcon(getClass(), "icons/JavaCup.gif"),
			"InternalFrame.icon"						, LookAndFeel.makeIcon(getClass(), "icons/JavaCup.gif"),
			"InternalFrame.iconifyIcon"				, LookAndFeel.makeIcon(getClass(), "icons/JavaCup.gif"),
			"InternalFrame.inactiveTitleBackground"	, ThreeDPrefs.desktopColor, 
			"InternalFrame.inactiveTitleForeground"	, ThreeDPrefs.disabledTextColor,
			"InternalFrame.maximizeIcon"				, LookAndFeel.makeIcon(getClass(), "icons/JavaCup.gif"),
			"InternalFrame.minimizeIcon"				, LookAndFeel.makeIcon(getClass(), "icons/JavaCup.gif"),
			"InternalFrame.titleFont"					, ThreeDPrefs.normalFont,
			// Label
			"Label.background"						, ThreeDPrefs.backgroundColor,
			"Label.border"							, null,
			"Label.disabledForeground"				, ThreeDPrefs.disabledTextColor,
			"Label.disabledShadow"					, table.get("controlShadow"),
			"Label.font"								, ThreeDPrefs.normalFont,
			"Label.foreground"						, ThreeDPrefs.textColor,
			// List
			"List.background"						, ThreeDPrefs.backgroundColor,
			"List.border"								, ThreeDPrefs.listBorder,
			"List.focusCellHighlightBorder"			, ThreeDPrefs.listFocusCellHighlightBorder,
			"List.font"								, ThreeDPrefs.normalFont,
			"List.foreground"							, ThreeDPrefs.textColor,
			"List.selectionBackground"				, ThreeDPrefs.highlightColor,
			"List.selectionForeground"				, ThreeDPrefs.textColor,
			// Menu
			"Menu.acceleratorFont"					, ThreeDPrefs.acceleratorFont,
			"Menu.acceleratorForeground"			, ThreeDPrefs.acceleratorColor,
			"Menu.acceleratorSelectionForeground"	, ThreeDPrefs.acceleratorColor,
			"Menu.arrowIcon"						, ThreeDPrefs.menuArrowIcon,
			"Menu.background"						, ThreeDPrefs.menuBackgroundColor,
			"Menu.border"							, ThreeDPrefs.menuBorder,
			"Menu.borderPainted"					, Boolean.TRUE,
			"Menu.checkIcon"						, null,
			"Menu.consumesTabs"					, Boolean.TRUE,
			"Menu.disabledForeground"				, ThreeDPrefs.disabledMenuTextColor,
			"Menu.font"								, ThreeDPrefs.normalFont,
			"Menu.foreground"						, ThreeDPrefs.menuTextColor,
			"Menu.margin"							, new InsetsUIResource(1,1,1,1),
			"Menu.selectionBackground"				, ThreeDPrefs.menuHighlightColor,
			"Menu.selectionForeground"				, ThreeDPrefs.menuTextColor,
			// MenuBar
			"MenuBar.background"					, ThreeDPrefs.menuBackgroundColor,
			"MenuBar.border"						, ThreeDPrefs.menuBarBorder,
			"MenuBar.font"							, ThreeDPrefs.normalFont,
			"MenuBar.foreground"					, ThreeDPrefs.menuTextColor,
			// Menu Item
			"MenuItem.acceleratorDelimiter"				, "\u00b7",
			"MenuItem.acceleratorFont"					, ThreeDPrefs.acceleratorFont,
			"MenuItem.acceleratorForeground"				, ThreeDPrefs.acceleratorColor,
			"MenuItem.acceleratorSelectionForeground"	, ThreeDPrefs.acceleratorColor,
			"MenuItem.arrowIcon"							, null,
			"MenuItem.background"						, ThreeDPrefs.menuBackgroundColor,
			"MenuItem.border"								, ThreeDPrefs.menuItemBorder,
			"MenuItem.borderPainted"						, Boolean.TRUE,
			"MenuItem.checkIcon"							, null,
			"MenuItem.disabledForeground"				, ThreeDPrefs.disabledMenuTextColor,
			"MenuItem.font"								, ThreeDPrefs.normalFont,
			"MenuItem.foreground"							, ThreeDPrefs.menuTextColor,
			"MenuItem.margin"								, new InsetsUIResource(1,1,1,1),
			"MenuItem.selectionBackground"				, ThreeDPrefs.menuHighlightColor,
			"MenuItem.selectionForeground"				, ThreeDPrefs.menuTextColor,
			// OptionPane
			"OptionPane.background"					, ThreeDPrefs.backgroundColor,
			"OptionPane.border"						, new BorderUIResource.EmptyBorderUIResource(10, 10, 10, 10),
			"OptionPane.buttonAreaBorder"			, new BorderUIResource.EmptyBorderUIResource(5,15,5,0),
			"OptionPane.errorIcon"						, LookAndFeel.makeIcon(getClass(), "icons/Error.gif"),
			"OptionPane.font"							, ThreeDPrefs.normalFont,
			"OptionPane.foreground"					, ThreeDPrefs.textColor,
			"OptionPane.informationIcon"				, LookAndFeel.makeIcon(getClass(), "icons/Inform.gif"),
			"OptionPane.messageAreaBorder"			, new BorderUIResource.EmptyBorderUIResource(0,0,0,0),
			"OptionPane.messageForeground"			, ThreeDPrefs.textColor,
			"OptionPane.minimumSize"					, new DimensionUIResource(262, 90),
			"OptionPane.questionIcon"					, LookAndFeel.makeIcon(getClass(), "icons/Question.gif"),
			"OptionPane.warningIcon"					, LookAndFeel.makeIcon(getClass(), "icons/Warn.gif"),
			// Panel
			"Panel.background"						, ThreeDPrefs.backgroundColor,
			"Panel.font"								, ThreeDPrefs.normalFont,
			"Panel.foreground"							, ThreeDPrefs.textColor,
			// PasswordField
			"PasswordField.background"				, ThreeDPrefs.textFieldBackgroundColor,
			"PasswordField.border"						, ThreeDPrefs.textComponentBorder,
			"PasswordField.caretBlinkRate"				, new Integer(ThreeDPrefs.caretBlinkRate),
			"PasswordField.caretForeground"			, ThreeDPrefs.caretColor,
			"PasswordField.font"						, ThreeDPrefs.normalFont,
			"PasswordField.foreground"				, ThreeDPrefs.textColor,
			"PasswordField.inactiveForeground"		, ThreeDPrefs.textColor,
			"PasswordField.focusInputMap"			, createTextFieldKeyBindings(),
			"PasswordField.margin"					, new InsetsUIResource(1,2,1,2),
			"PasswordField.selectionBackground"		, ThreeDPrefs.highlightColor,
			"PasswordField.selectionForeground"		, ThreeDPrefs.textColor,
			// PopupMenu
			"PopupMenu.background"					, ThreeDPrefs.menuBackgroundColor,
			"PopupMenu.border"						, ThreeDPrefs.popupMenuBorder,          
			"PopupMenu.font"							, ThreeDPrefs.normalFont,
			"PopupMenu.foreground"					, ThreeDPrefs.textColor,
			// Progress Bars
			"ProgressBar.background"					, ThreeDPrefs.backgroundColor,
			"ProgressBar.border"						, ThreeDPrefs.progressBarBorder,
			"ProgressBar.cellLength"					, new Integer(1),
			"ProgressBar.cellSpacing"					, new Integer(0),
			"ProgressBar.font"							, ThreeDPrefs.normalFont,
			"ProgressBar.foreground"					, ThreeDPrefs.highlightColor, 
			"ProgressBar.selectionBackground"			, ThreeDPrefs.textColor, 
			"ProgressBar.selectionForeground"			, ThreeDPrefs.textColor,
			// RadioButton
			"RadioButton.background"					, ThreeDPrefs.backgroundColor,
			"RadioButton.border"						, BorderFactory.createEmptyBorder(1,1,1,1),
			"RadioButton.font"							, ThreeDPrefs.normalFont,
			"RadioButton.foreground"					, ThreeDPrefs.textColor,
			"RadioButton.icon"							, ThreeDPrefs.radioButtonIcon,
			"RadioButton.margin"						, new InsetsUIResource(3,5,3,5),
			"RadioButton.textIconGap"					, new Integer(4),
			"RadioButton.textShiftOffset"				, new Integer(0),
			// RadioButtonMenuItem
			"RadioButtonMenuItem.acceleratorFont"					, ThreeDPrefs.acceleratorFont,
			"RadioButtonMenuItem.acceleratorForeground"				, ThreeDPrefs.acceleratorColor,
			"RadioButtonMenuItem.acceleratorSelectionForeground"	, ThreeDPrefs.acceleratorColor,
			"RadioButtonMenuItem.arrowIcon"							, null,
			"RadioButtonMenuItem.background"						, ThreeDPrefs.menuBackgroundColor,
			"RadioButtonMenuItem.border"								, ThreeDPrefs.menuItemBorder,
			"RadioButtonMenuItem.borderPainted"						, Boolean.TRUE,
			"RadioButtonMenuItem.checkIcon"							, new ThreeDIcons.RadioButtonMenuIcon(),
			"RadioButtonMenuItem.disabledForeground"				, ThreeDPrefs.disabledMenuTextColor,
			"RadioButtonMenuItem.font"								, ThreeDPrefs.normalFont,
			"RadioButtonMenuItem.foreground"						, ThreeDPrefs.menuTextColor,
			"RadioButtonMenuItem.margin"							, new InsetsUIResource(3,5,3,5),
			"RadioButtonMenuItem.selectionBackground"			, ThreeDPrefs.highlightColor,
			"RadioButtonMenuItem.selectionForeground"			, ThreeDPrefs.menuTextColor,
			// ScrollBar
			"ScrollBar.background"			, ThreeDPrefs.backgroundColor,
			"ScrollBar.border"				, null,
			"ScrollBar.foreground"			, ThreeDPrefs.textColor,
			"ScrollBar.lighShadow"			, Color.red,
			"ScrollBar.maximumThumbSize"	, new Dimension(1024,1024),
			"ScrollBar.minimumThumbSize"	, new Dimension(17,17),
			"ScrollBar.thumb"				, Color.red,
			"ScrollBar.thumbDarkShadow"	, Color.red,
			"ScrollBar.thumbHighlight"		, Color.red,
			"ScrollBar.track"					, Color.red,
			"ScrollBar.trackHighlight"			, Color.red,
			// ScrollPane
			"ScrollPane.background"			, ThreeDPrefs.backgroundColor,
			"ScrollPane.border"				, ThreeDPrefs.scrollPaneBorder,
			"ScrollPane.font"					, ThreeDPrefs.normalFont,
			"ScrollPane.foreground"			, ThreeDPrefs.textColor,
			"ScrollPane.viewportBorder"		, ThreeDPrefs.scrollPaneViewportBorder,
			// Separator
			"Separator.background"			, ThreeDPrefs.lightEdgeColor,
			"Separator.foreground"			, ThreeDPrefs.darkEdgeColor,
			// Slider
			"Slider.background"				, ThreeDPrefs.backgroundColor,
			"Slider.border"					, null,
			"Slider.focus"					, ThreeDPrefs.focusColor,
			"Slider.focusInsets"				, new InsetsUIResource(1,1,1,1),
			"Slider.foreground"				, ThreeDPrefs.textColor,
			"Slider.highlight"					, ThreeDPrefs.lightEdgeColor,
			"Slider.shadow"					, ThreeDPrefs.darkEdgeColor,
			// SplitPane
			"SplitPane.background"			, ThreeDPrefs.backgroundColor,
			"SplitPane.border"				, ThreeDPrefs.splitPaneBorder,
			"SplitPane.dividerSize"			, new Integer(ThreeD_SplitPaneDivider._getDividerSize()),
			"SplitPane.hightlight"				, ThreeDPrefs.lightEdgeColor,
			"SplitPane.shadow"				, ThreeDPrefs.darkEdgeColor,
			"SplitPaneDivider.border"			, null,
			// TabbedPane
			"TabbedPane.background"			, ThreeDPrefs.backgroundColor,
			"TabbedPane.contentBorderInsets"	, new InsetsUIResource(4,4,4,4),
			"TabbedPane.darkShadow"			, ThreeDPrefs.darkEdgeColor,
			"TabbedPane.focus"					, ThreeDPrefs.focusColor,
			"TabbedPane.font"					, ThreeDPrefs.normalFont,
			"TabbedPane.foreground"				, ThreeDPrefs.textColor,
			"TabbedPane.highlight"				, ThreeDPrefs.lightEdgeColor,
			"TabbedPane.lightHighlight"			, ThreeDPrefs.lightEdgeColor,
			"TabbedPane.selectedTabPadInsets"	, new InsetsUIResource(4,4,2,4),
			"TabbedPane.shadow"				, ThreeDPrefs.darkEdgeColor,
			"TabbedPane.tabAreaInsets"			, new InsetsUIResource(4,10,0,10),
			"TabbedPane.tabInsets"				, new InsetsUIResource(4,4,4,4),
			"TabbedPane.tabRunOverlay"			, new Integer(2),
			"TabbedPane.textIconGap"			, new Integer(4),
			// Table
			"Table.background"					, ThreeDPrefs.backgroundColor,
			"Table.focusCellBackground"			, ThreeDPrefs.backgroundColor,
			"Table.focusCellForeground"			, ThreeDPrefs.textColor,
			"Table.focusCellHighlightBorder"		, new BorderUIResource.LineBorderUIResource(ThreeDPrefs.focusColor),
			"Table.font"							, ThreeDPrefs.normalFont,
			"Table.foreground"					, ThreeDPrefs.textColor,
			"Table.gridColor"						, ThreeDPrefs.backgroundColor.darker(),
			"Table.scrollPaneBorder"				, ThreeDPrefs.tableBorder,
			"Table.selectionBackground"			, ThreeDPrefs.highlightColor,
			"Table.selectionForeground"			, ThreeDPrefs.textColor,
			"TableHeader.background"			, ThreeDPrefs.backgroundColor,
			"TableHeader.cellBorder"				, ThreeDPrefs.tableHeaderBorder,
			"TableHeader.font"					, ThreeDPrefs.normalFont,
			"TableHeader.foreground"				, ThreeDPrefs.textColor,
			// TextArea
			"TextArea.background"			, ThreeDPrefs.textFieldBackgroundColor,
			"TextArea.border"				, ThreeDPrefs.textComponentBorder,
			"TextArea.caretBlinkRate"		, new Integer(ThreeDPrefs.caretBlinkRate),
			"TextArea.caretForeground"		, ThreeDPrefs.caretColor,
			"TextArea.font"					, ThreeDPrefs.normalFont,
			"TextArea.foreground"			, ThreeDPrefs.textColor,
			"TextArea.inactiveForeground"		, ThreeDPrefs.textColor,
			"TextArea.focusInputMap"		, createTextEditorKeyBindings(),
			"TextArea.margin"				, new InsetsUIResource(2,2,2,2),
			"TextArea.selectionBackground"	, ThreeDPrefs.highlightColor,
			"TextArea.selectionForeground"	, ThreeDPrefs.textColor,
			// TextField
			"TextField.background"			, ThreeDPrefs.textFieldBackgroundColor,
			"TextField.border"				, ThreeDPrefs.textComponentBorder,
			"TextField.caretBlinkRate"		, new Integer(ThreeDPrefs.caretBlinkRate),
			"TextField.caretForeground"		, ThreeDPrefs.caretColor,
			"TextField.font"					, ThreeDPrefs.normalFont,
			"TextField.foreground"			, ThreeDPrefs.textColor,
			"TextField.inactiveForeground"	, ThreeDPrefs.textColor,
			"TextField.focusInputMap"		, createTextFieldKeyBindings(),
			"TextField.margin"				, new InsetsUIResource(1,2,1,2),
			"TextField.selectionBackground"	, ThreeDPrefs.highlightColor,
			"TextField.selectionForeground"	, ThreeDPrefs.textColor,
			// TextPane
			"TextPane.background"				, ThreeDPrefs.textFieldBackgroundColor,
			"TextPane.border"					, ThreeDPrefs.textComponentBorder,
			"TextPane.caretBlinkRate"			, new Integer(ThreeDPrefs.caretBlinkRate),
			"TextPane.caretForeground"			, ThreeDPrefs.caretColor,
			"TextPane.font"						, ThreeDPrefs.normalFont,
			"TextPane.foreground"				, ThreeDPrefs.textColor,
			"TextPane.inactiveForeground"		, ThreeDPrefs.textColor,
			"TextPane.focusInputMap"			, createTextEditorKeyBindings(),
			"TextPane.margin"					, new InsetsUIResource(2,2,2,2),
			"TextPane.selectionBackground"		, ThreeDPrefs.highlightColor,
			"TextPane.selectionForeground"		, ThreeDPrefs.textColor,
			// Titled Border
			"TitledBorder.border"					, ThreeDPrefs.titledBorder,
			"TitledBorder.font"					, ThreeDPrefs.titledBorderFont,
			"TitledBorder.titleColor"				, ThreeDPrefs.titledBorderTextColor,
			// ToggleButton
			"ToggleButton.background"			, ThreeDPrefs.backgroundColor,
			"ToggleButton.border"				, ThreeDPrefs.toggleButtonBorder,
			"ToggleButton.font"					, ThreeDPrefs.normalFont,
			"ToggleButton.foreground"			, ThreeDPrefs.textColor,
			"ToggleButton.margin"				, new InsetsUIResource(3,5,3,5),
			"ToggleButton.textIconGap"			, new Integer(4),
			"ToggleButton.textShiftOffset"			, new Integer(0),
			// ToolBar
			"ToolBar.background"				, ThreeDPrefs.backgroundColor,
			"ToolBar.border"						, ThreeDPrefs.toolBarBorder,
			"ToolBar.dockingBackground"			, ThreeDPrefs.backgroundColor,
			"ToolBar.dockingForeground"			, ThreeDPrefs.textColor,
			"ToolBar.floatingBackground"			, ThreeDPrefs.backgroundColor,
			"ToolBar.floatingForeground"			, ThreeDPrefs.textColor,
			"ToolBar.font"						, ThreeDPrefs.normalFont,
			"ToolBar.foreground"					, ThreeDPrefs.textColor,
			"ToolBar.separatorSize"				, new DimensionUIResource(10,10),
			// ToolTip
			"ToolTip.background"					, ThreeDPrefs.toolTipBackgroundColor,
			"ToolTip.border"						, new BorderUIResource.LineBorderUIResource(Color.black),
			"ToolTip.font"						, ThreeDPrefs.normalFont,
			"ToolTip.foreground"					, ThreeDPrefs.textColor,
			// Tree
			"Tree.background"					, ThreeDPrefs.backgroundColor,
			"Tree.changeSelectionWithFocus"		, Boolean.TRUE,
			"Tree.closedIcon"					, LookAndFeel.makeIcon(getClass(), "icons/TreeClosed.gif"),
			"Tree.collapsedIcon"					, null,
			"Tree.drawsFocusBorderAroundIcon"	, Boolean.FALSE,
			"Tree.editorBorder"					, new BorderUIResource.LineBorderUIResource(Color.black),
			"Tree.expandedIcon"					, null,
			"Tree.font"							, ThreeDPrefs.normalFont,
			"Tree.foreground"						, ThreeDPrefs.textColor,
			"Tree.hash"							, Color.black,
			"Tree.leafIcon"						, LookAndFeel.makeIcon(getClass(), "icons/TreeLeaf.gif"),
			"Tree.leftChildIndent"					, new Integer(7),
			"Tree.openIcon"						, LookAndFeel.makeIcon(getClass(), "icons/TreeOpen.gif"),
			"Tree.rightChildIndent"				, new Integer(13),
			"Tree.rowHeight"						, new Integer(16),
			"Tree.scrollsOnExpand"				, Boolean.TRUE,
			"Tree.selectionBackground"			, ThreeDPrefs.highlightColor,
			"Tree.selectionBorderColor"			, ThreeDPrefs.focusColor,
			"Tree.selectionForeground"			, ThreeDPrefs.textColor,
			"Tree.textBackground"				, ThreeDPrefs.backgroundColor,
			"Tree.textForeground"				, ThreeDPrefs.textColor,
			// ViewPort
			"Viewport.background"				, ThreeDPrefs.backgroundColor,
			"Viewport.font"						, ThreeDPrefs.normalFont,
			"Viewport.foreground"					, ThreeDPrefs.textColor,
			};
		
		table.putDefaults(defaults);
	}

	/**
	* Load the SystemColors into the defaults table.  The keys
	* for SystemColor defaults are the same as the names of
	* the public fields in SystemColor.
	* 
	*/

	protected void initSystemColorDefaults(UIDefaults table)  {
		super.initSystemColorDefaults(table);
		
		Object[] systemColors = {
			"desktop"	, ThreeDPrefs.desktopColor, 		/* Color of the desktop background */
			"window"	, ThreeDPrefs.backgroundColor, 	/* Default color for the interior of windows */
		};
		
		for (int i = 0; i < systemColors.length; i += 2) table.put((String)systemColors[i], systemColors[i + 1]);
		
	}

	public boolean isNativeLookAndFeel()  {
		return false;
	}

	public boolean isSupportedLookAndFeel()  {
		return true;
	}

}
