/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.threeD;

import java.awt.*;
import java.io.*;
import java.util.zip.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import java.util.*;

public class ThreeDPrefs {
	// Konstanten
	private static final String FILENAME 			= System.getProperty("user.home") + File.separator + ".3dlf";
	
	// Misc
	private static Border marginBorder 			= new BasicBorders.MarginBorder();
	
	// System
	static	Font	normalFont					= null;
	static	Color	backgroundColor				= null;
	static	Color	textColor					= null;
	static	Color	highlightColor				= null;
	static	Color	focusColor					= null;
	static	Color	disableColor					= null;
	static	Color	disabledTextColor			= null;
	
	// Borders
	static	Color	lightEdgeColor				= null;
	static	Color	darkEdgeColor				= null;
	static	Font	titledBorderFont				= null;
	static	Color	titledBorderTextColor			= null;
	static	int		buttonBorderType1			= 0;
	static	int 		buttonBorderType2			= 0;
	static	int		checkBoxBorderType1		= 0;
	static	int		checkBoxBorderType2		= 0;
	static	int		comboBoxBorderType		= 0;
	static	int		listBorderType				= 0;
	static	int		menuBorderType				= 0;
	static	int		menuBarBorderType			= 0;
	static	int		menuItemBorderType			= 0;
	static	int		popupMenuBorderType		= 0;
	static	int		progressBarBorderType		= 0;
	static	int		radioButtonBorderType1		= 0;
	static	int 		radioButtonBorderType2		= 0;
	static	int 		scrollBarThumbBorderType	= 0;
	static	int		scrollBarTrackBorderType		= 0;
	static	int 		sliderThumbBorderType		= 0;
	static	int		sliderTrackBorderType		= 0;
	static	int		tableBorderType				= 0;
	static	int		tableHeaderBorderType		= 0;
	static	int		toggleButtonBorderType1		= 0;
	static	int 		toggleButtonBorderType2		= 0;
	static	int		textComponentBorderType1	= 0;
	static	int		textComponentBorderType2	= 0;
	static	int		treeBorderType				= 0;
	
	// Desktop
	static	Color	desktopColor				= null;
	static	Font	internalFrameTitleFont		= null;
	
	// Menus
	static	Color	menuBackgroundColor		= null;
	static	Color	menuTextColor				= null;
	static	Color	menuHighlightColor			= null;
	static	Color	disabledMenuTextColor		= null;
	static	Font	acceleratorFont				= null;
	static	Color	acceleratorColor				= null;
	
	// ScrollBar
	static	Color	scrollBarTrackColor			= null;
	static	Color	scrollBarThumbColor			= null;
	
	// Slider
	static	Color	sliderTrackColor				= null;
	static	Color	sliderThumbColor			= null;
	
	// TabbedPane
	static	Color	tabbedPaneBackgroundColor	= null;
	
	// TextComponents
	static	Color	textFieldBackgroundColor		= null;
	static	Color	caretColor					= null;
	static	int		caretBlinkRate				= 0;
	
	// ToolTips
	static	Color	toolTipBackgroundColor		= null;
	
	// Icons
	static	Icon	checkBoxIcon				= new ThreeDIcons.CheckBoxIcon();
	static	Icon	checkBoxMenuIcon			= new ThreeDIcons.CheckBoxMenuIcon();
	static	Icon	radioButtonIcon				= new ThreeDIcons.RadioButtonIcon();
	static	Icon	radioButtonMenuIcon			= new ThreeDIcons.RadioButtonMenuIcon();
	static	Icon	menuArrowIcon				= new ThreeDIcons.ArrowIcon(ThreeDIcons.RIGHT, 7);
	static	Icon	internalFrameCloseIcon		= new ThreeDIcons.ArrowIcon(ThreeDIcons.LEFT, 7);
	static	Icon	internalFrameIconifyIcon		= new ThreeDIcons.ArrowIcon(ThreeDIcons.DOWN, 7);
	static	Icon	internalFrameSizeIcon		= new ThreeDIcons.ArrowIcon(ThreeDIcons.UP, 7);
	
	// Borders
	static	Border	buttonBorder					= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.ButtonBorder(), marginBorder);
	static	Border	toggleButtonBorder			= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.ToggleButtonBorder(), marginBorder);
	static	Border	progressBarBorder			= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.ProgressBarBorder(), marginBorder);
	static	Border	titledBorder					= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.TitledBorder(), marginBorder);
	static	Border	toolBarBorder				= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.ToolBarBorder(), marginBorder);
	static	Border	menuBarBorder				= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.MenuBarBorder(), marginBorder);
	static	Border	menuBorder					= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.MenuBorder(), marginBorder);
	static	Border	menuItemBorder				= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.MenuItemBorder(), marginBorder);
	static	Border	popupMenuBorder			= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.PopupMenuBorder(), marginBorder);
	static	Border	textComponentBorder		= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.TextComponentBorder(), marginBorder);
	static	Border	scrollPaneBorder				= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.ScrollPaneBorder(), marginBorder);
	static	Border	scrollPaneViewportBorder		= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.ScrollPaneViewportBorder(), marginBorder);
	static	Border	splitPaneBorder				= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.SplitPaneBorder(), marginBorder);
	static	Border	listBorder					= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.ListBorder(), marginBorder);
	static	Border	listFocusCellHighlightBorder	= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.ListFocusCellHighlightBorder(), marginBorder);
	static	Border	tableBorder					= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.TableBorder(), marginBorder);
	static	Border	tableHeaderBorder			= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.TableHeaderBorder(), marginBorder);
	static	Border	desktopIconBorder			= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.DesktopIconBorder(), marginBorder);
	static	Border	internalFrameBorder			= new BorderUIResource.CompoundBorderUIResource(
														new ThreeDBorders.InternalFrameBorder(), marginBorder);
	
	static {
		try
			{
			resetToLastSaved();
			}
		catch (IOException x) { resetToDefaultValues(); }
	}

	static ColorUIResource decodeColor(String s, Color c)  {
		try
			{
			StringTokenizer tok = new StringTokenizer(s, ", ");
			int r = Integer.parseInt(tok.nextToken());
			int g = Integer.parseInt(tok.nextToken());
			int b = Integer.parseInt(tok.nextToken());
			return new ColorUIResource(r,g,b);
			}
		catch (Exception x) { return new ColorUIResource(c); }
	}

	static FontUIResource decodeFont(String s, Font f)  {
		try
			{
			StringTokenizer tok = new StringTokenizer(s, ", ");
			String name = tok.nextToken();
			int size = Integer.parseInt(tok.nextToken());
			int style = Integer.parseInt(tok.nextToken());
			return new FontUIResource(name, style, size);
			}
		catch (Exception x) { return new FontUIResource(f); }
	}

	static int decodeInt(String s, int i)  {
		try
			{
			return Integer.parseInt(s);
			}
		catch (Exception x) { System.out.println(x); return i; }
	}

	static String encodeColor(Color c)  {
		return c.getRed() + "," + c.getGreen() + "," + c.getBlue();
	}

	static String encodeFont(Font f)  {
		return f.getName() + "," + f.getSize() + "," + f.getStyle();
	}

	static void load() throws IOException {
		load(FILENAME);
	}

	static void load(String filename) throws IOException {
		// Stream ffnen
		FileInputStream in = new FileInputStream(filename);
		
		// Properties lesen
		Properties props = new Properties();
		props.load(in);
		
		// Stream schlieen
		in.close();
		
		// Schon mal Default-Werte setzen
		resetToDefaultValues();
		
		// System
		normalFont 						= decodeFont(props.getProperty("normalFont"), normalFont);
		backgroundColor 				= decodeColor(props.getProperty("backgroundColor"), backgroundColor);
		textColor						= decodeColor(props.getProperty("textColor"), textColor);
		highlightColor 					= decodeColor(props.getProperty("highlightColor"), highlightColor);
		focusColor 						= decodeColor(props.getProperty("focusColor"), focusColor);
		disableColor 					= decodeColor(props.getProperty("disableColor"), disableColor);
		disabledTextColor 				= decodeColor(props.getProperty("disabledTextColor"), disabledTextColor);
		
		// Borders
		lightEdgeColor 					= decodeColor(props.getProperty("lightEdgeColor"), lightEdgeColor);
		darkEdgeColor 					= decodeColor(props.getProperty("darkEdgeColor"), darkEdgeColor);
		buttonBorderType1				= decodeInt(props.getProperty("buttonBorderType1"), buttonBorderType1);
		buttonBorderType2				= decodeInt(props.getProperty("buttonBorderType2"), buttonBorderType2);
		checkBoxBorderType1			= decodeInt(props.getProperty("checkBoxBorderType1"), checkBoxBorderType1);
		checkBoxBorderType2			= decodeInt(props.getProperty("checkBoxBorderType2"), checkBoxBorderType2);
		comboBoxBorderType			= decodeInt(props.getProperty("comboBoxBorderType"), comboBoxBorderType);
		listBorderType					= decodeInt(props.getProperty("listBorderType"), listBorderType);
		menuBorderType					= decodeInt(props.getProperty("menuBorderType"), menuBorderType);
		menuBarBorderType				= decodeInt(props.getProperty("menuBarBorderType"), menuBarBorderType);
		menuItemBorderType				= decodeInt(props.getProperty("menuItemBorderType"), menuItemBorderType);
		popupMenuBorderType			= decodeInt(props.getProperty("popupMenuBorderType"), popupMenuBorderType);
		progressBarBorderType			= decodeInt(props.getProperty("progressBarBorderType"), progressBarBorderType);
		radioButtonBorderType1			= decodeInt(props.getProperty("radioButtonBorderType1"), radioButtonBorderType1);
		radioButtonBorderType2			= decodeInt(props.getProperty("radioButtonBorderType2"), radioButtonBorderType2);
		scrollBarThumbBorderType		= decodeInt(props.getProperty("scrollBarThumbBorderType"), scrollBarThumbBorderType);
		scrollBarTrackBorderType			= decodeInt(props.getProperty("scrollBarTrackBorderType"), scrollBarTrackBorderType);
		sliderThumbBorderType			= decodeInt(props.getProperty("sliderThumbBorderType"), sliderThumbBorderType);
		sliderTrackBorderType			= decodeInt(props.getProperty("sliderTrackBorderType"), sliderTrackBorderType);
		tableBorderType					= decodeInt(props.getProperty("tableBorderType"), tableBorderType);
		tableHeaderBorderType			= decodeInt(props.getProperty("tableHeaderBorderType"), tableHeaderBorderType);
		textComponentBorderType1		= decodeInt(props.getProperty("textComponentBorderType1"), textComponentBorderType1);
		textComponentBorderType2		= decodeInt(props.getProperty("textComponentBorderType2"), textComponentBorderType2);
		toggleButtonBorderType1			= decodeInt(props.getProperty("toggleButtonBorderType1"), toggleButtonBorderType1);
		toggleButtonBorderType2			= decodeInt(props.getProperty("toggleButtonBorderType2"), toggleButtonBorderType2);
		treeBorderType					= decodeInt(props.getProperty("treeBorderType"), treeBorderType);
		
		// Desktop
		desktopColor 					= decodeColor(props.getProperty("desktopColor"), desktopColor);
		internalFrameTitleFont 			= decodeFont(props.getProperty("internalFrameTitleFont"), internalFrameTitleFont);
		
		// Menus
		menuBackgroundColor 			= decodeColor(props.getProperty("menuBackgroundColor"), menuBackgroundColor);
		menuTextColor 					= decodeColor(props.getProperty("menuTextColor"), menuTextColor);
		menuHighlightColor 				= decodeColor(props.getProperty("menuHighlightColor"), menuHighlightColor);
		disabledMenuTextColor 			= decodeColor(props.getProperty("disabledMenuTextColor"), disabledMenuTextColor);
		acceleratorFont 					= decodeFont(props.getProperty("acceleratorFont"), acceleratorFont);
		acceleratorColor 				= decodeColor(props.getProperty("acceleratorColor"), acceleratorColor);
		
		// ScrollBar
		scrollBarTrackColor 				= decodeColor(props.getProperty("scrollBarTrackColor"), scrollBarTrackColor);
		scrollBarThumbColor 				= decodeColor(props.getProperty("scrollBarThumbColor"), scrollBarThumbColor);
		
		// Slider
		sliderThumbColor 				= decodeColor(props.getProperty("sliderThumbColor"), sliderThumbColor);
		
		// TabbedPane
		tabbedPaneBackgroundColor 		= decodeColor(props.getProperty("tabbedPaneBackgroundColor"), tabbedPaneBackgroundColor);
		
		// TextComponents
		textFieldBackgroundColor 		= decodeColor(props.getProperty("textFieldBackgroundColor"), textFieldBackgroundColor);
		caretColor 						= decodeColor(props.getProperty("caretColor"), caretColor);
		caretBlinkRate 					= decodeInt(props.getProperty("caretBlinkRate"), caretBlinkRate);
		
		// TitledBorder
		titledBorderFont 					= decodeFont(props.getProperty("titledBorderFont"), titledBorderFont);
		titledBorderTextColor 				= decodeColor(props.getProperty("titledBorderTextColor"), titledBorderTextColor);
		
		// ToolTips
		toolTipBackgroundColor 			= decodeColor(props.getProperty("toolTipBackgroundColor"), toolTipBackgroundColor);
	}

	static void resetToDefaultValues()  {
		// System
		normalFont					= new FontUIResource("SansSerif", Font.PLAIN, 12);
		backgroundColor			= new ColorUIResource(170,170,170);
		textColor					= new ColorUIResource(0,0,0);
		highlightColor				= new ColorUIResource(102,136,187);
		focusColor					= new ColorUIResource(238,68,68);
		disableColor				= new ColorUIResource(0,0,0);
		disabledTextColor			= new ColorUIResource(100,100,100);
		
		// Borders
		lightEdgeColor				= new ColorUIResource(255,255,255);
		darkEdgeColor				= new ColorUIResource(0,0,0);
		buttonBorderType1			= ThreeDUtils.RAISED_THIN;
		buttonBorderType2			= ThreeDUtils.LOWERED_THIN;
		checkBoxBorderType1		= ThreeDUtils.RAISED_THIN;
		checkBoxBorderType2		= ThreeDUtils.LOWERED_THIN;
		comboBoxBorderType		= ThreeDUtils.RAISED_THIN;
		listBorderType				= ThreeDUtils.RAISED_THIN;
		menuBorderType				= ThreeDUtils.LOWERED_THIN;
		menuBarBorderType			= ThreeDUtils.RAISED_THIN;
		menuItemBorderType			= ThreeDUtils.LOWERED_THIN;
		popupMenuBorderType		= ThreeDUtils.RAISED_THIN;
		progressBarBorderType		= ThreeDUtils.LOWERED_THIN;
		radioButtonBorderType1		= ThreeDUtils.RAISED_THIN;
		radioButtonBorderType2		= ThreeDUtils.LOWERED_THIN;
		scrollBarThumbBorderType	= ThreeDUtils.RAISED_THIN;
		scrollBarTrackBorderType		= ThreeDUtils.ETCHED_OUT_THIN;
		sliderThumbBorderType		= ThreeDUtils.RAISED_THIN;
		sliderTrackBorderType		= ThreeDUtils.LOWERED_THIN;
		tableBorderType				= ThreeDUtils.RAISED_THIN;
		tableHeaderBorderType		= ThreeDUtils.RAISED_THIN;
		textComponentBorderType1 	= ThreeDUtils.ETCHED_OUT_THIN;
		textComponentBorderType2	= ThreeDUtils.LOWERED_THIN;
		toggleButtonBorderType1		= ThreeDUtils.RAISED_THIN;
		toggleButtonBorderType2		= ThreeDUtils.LOWERED_THIN;
		treeBorderType				= ThreeDUtils.RAISED_THIN;
		
		// Desktop
		desktopColor				= new ColorUIResource(165,165,165);
		internalFrameTitleFont		= new FontUIResource("SansSerif", Font.PLAIN, 10);
		
		// Menus
		menuBackgroundColor		= new ColorUIResource(155,155,155);
		menuTextColor				= new ColorUIResource(0,0,0);
		menuHighlightColor			= new ColorUIResource(102,136,187);
		disabledMenuTextColor		= new ColorUIResource(100,100,100);
		acceleratorFont				= new FontUIResource("Dialog", Font.PLAIN, 10);
		acceleratorColor				= new ColorUIResource(102,102,153);
		
		// ScrollBar
		scrollBarTrackColor			= new ColorUIResource(170,170,170);
		scrollBarThumbColor			= new ColorUIResource(102,136,187);
		
		// Slider
		sliderTrackColor				= new ColorUIResource(155,155,155);
		sliderThumbColor			= new ColorUIResource(102,136,187);
		
		// TabbedPane
		tabbedPaneBackgroundColor	= new ColorUIResource(155,155,155);
		
		// TextComponents
		textFieldBackgroundColor	= new ColorUIResource(200,200,200);
		caretColor					= new ColorUIResource(200,0,0);
		caretBlinkRate				= 250;
		
		// TitledBorder
		titledBorderFont				= new FontUIResource("SansSerif", Font.ITALIC, 12);
		titledBorderTextColor			= new ColorUIResource(255,255,255);
		
		// ToolTips
		toolTipBackgroundColor		= new ColorUIResource(220,220,220);
	}

	static void resetToLastSaved() throws IOException {
		load(FILENAME);
	}

	static void save()  {
		save(FILENAME);
		
	}

	static void save(String filename)  {
		Properties props = new Properties();
		
		// Version
		props.put("version", "1");
		
		// System
		props.put("normalFont"				, encodeFont(normalFont));
		props.put("backgroundColor"			, encodeColor(backgroundColor));
		props.put("textColor"					, encodeColor(textColor));
		props.put("highlightColor"			, encodeColor(highlightColor));
		props.put("focusColor"				, encodeColor(focusColor));
		props.put("disableColor"				, encodeColor(disableColor));
		props.put("disabledTextColor"		, encodeColor(disabledTextColor));
		
		// Borders
		props.put("lightEdgeColor"			, encodeColor(lightEdgeColor));
		props.put("darkEdgeColor"			, encodeColor(darkEdgeColor));
		props.put("buttonBorderType1"		, "" + buttonBorderType1);
		props.put("buttonBorderType2"		, "" + buttonBorderType2);
		props.put("checkBoxBorderType1"		, "" + checkBoxBorderType1);
		props.put("checkBoxBorderType2"		, "" + checkBoxBorderType2);
		props.put("comboBoxBorderType"		, "" + comboBoxBorderType);
		props.put("listBorderType"				, "" + listBorderType);
		props.put("menuBorderType"			, "" + menuBorderType);
		props.put("menuBarBorderType"		, "" + menuBarBorderType);
		props.put("menuItemBorderType"		, "" + menuItemBorderType);
		props.put("popupMenuBorderType"	, "" + popupMenuBorderType);
		props.put("progressBarBorderType"	, "" + progressBarBorderType);
		props.put("radioButtonBorderType1"	, "" + radioButtonBorderType1);
		props.put("radioButtonBorderType2"	, "" + radioButtonBorderType2);
		props.put("scrollBarTrackBorderType"	, "" + scrollBarTrackBorderType);
		props.put("scrollBarThumbBorderType"	, "" + scrollBarThumbBorderType);
		props.put("sliderTrackBorderType"		, "" + sliderTrackBorderType);
		props.put("sliderThumbBorderType"	, "" + sliderThumbBorderType);
		props.put("tableBorderType"			, "" + tableBorderType);
		props.put("tableHeaderBorderType"	, "" + tableHeaderBorderType);
		props.put("textComponentBorderType1"	, "" + textComponentBorderType1);
		props.put("textComponentBorderType2"	, "" + textComponentBorderType2);
		props.put("toggleButtonBorderType1"	, "" + toggleButtonBorderType1);
		props.put("toggleButtonBorderType2"	, "" + toggleButtonBorderType2);
		props.put("treeBorderType"			, "" + treeBorderType);
		
		// Desktop
		props.put("desktopColor"			, encodeColor(desktopColor));
		props.put("internalFrameTitleFont"	, encodeFont(internalFrameTitleFont));
		
		// Menus
		props.put("menuBackgroundColor"		, encodeColor(menuBackgroundColor));
		props.put("menuTextColor"			, encodeColor(menuTextColor));
		props.put("menuHighlightColor"		, encodeColor(menuHighlightColor));
		props.put("disabledMenuTextColor"	, encodeColor(disabledMenuTextColor));
		props.put("acceleratorFont"			, encodeFont(acceleratorFont));
		props.put("acceleratorColor"			, encodeColor(acceleratorColor));
		
		// ScrollBar
		props.put("scrollBarTrackColor"		, encodeColor(scrollBarTrackColor));
		props.put("scrollBarThumbColor"		, encodeColor(scrollBarThumbColor));
		
		// Slider
		props.put("sliderTrackColor"			, encodeColor(sliderTrackColor));
		props.put("sliderThumbColor"			, encodeColor(sliderThumbColor));
		
		// TabbedPane
		props.put("tabbedPaneBackgroundColor"	, encodeColor(tabbedPaneBackgroundColor));
		
		// TextComponents
		props.put("textFieldBackgroundColor"	, encodeColor(textFieldBackgroundColor));
		props.put("caretColor"				, encodeColor(caretColor));
		props.put("caretBlinkRate"				, "" + caretBlinkRate);
		
		// TitledBorder
		props.put("titledBorderFont"			, encodeFont(titledBorderFont));
		props.put("titledBorderTextColor"		, encodeColor(titledBorderTextColor));
		
		// ToolTips
		props.put("toolTipBackgroundColor"	, encodeColor(toolTipBackgroundColor));
		
		// Und rausschreiben
		try
			{
			FileOutputStream out = new FileOutputStream(filename);
			props.store(out, "Please do not edit manually!");
			out.close();
			}
		catch (IOException x) { System.err.println(x); }
	}

}
