/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.threeD;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import javax.swing.ImageIcon;
import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;

final class ThreeDUtils {
	public static final int 	BORDER_TYPES		= 12;
	
	public static final int 	LINE_LIGHT				= 0;
	public static final int 	LINE_DARK				= 1;
	public static final int 	RAISED_THIN			= 2;
	public static final int 	LOWERED_THIN		= 3;
	public static final int 	RAISED_THICK			= 4;
	public static final int 	LOWERED_THICK		= 5;
	public static final int 	ETCHED_OUT_THIN		= 6;
	public static final int 	ETCHED_IN_THIN		= 7;
	public static final int 	ETCHED_OUT_THICK	= 8;
	public static final int 	ETCHED_IN_THICK		= 9;
	public static final int	QT_IN					= 10;
	public static final int	QT_OUT				= 11;
	
	private static final Insets 	zeroInsets 		= new Insets(0,0,0,0);
	private static final Insets 	oneInsets 		= new Insets(1,1,1,1);
	private static final Insets 	twoInsets 		= new Insets(2,2,2,2);
	private static final Insets 	fourInsets 		= new Insets(4,4,4,4);
	
	public static final void drawBorder(int type, Graphics g, int x, int y, int w, int h)  {
		g.translate(x,y);
		
		switch (type)
			{
			case LINE_LIGHT:
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawRect(0,0,w-1,h-1);
				break;
			case LINE_DARK:
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawRect(0,0,w-1,h-1);
				break;
			case RAISED_THIN:
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawLine(0,0,0,h-2);
				g.drawLine(0,0,w-2,0);
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawLine(w-1, h-1, w-1, 1);
				g.drawLine(w-1, h-1, 1, h-1);
				break;
			case LOWERED_THIN:
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawLine(0,0,0,h-2);
				g.drawLine(0,0,w-2,0);
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawLine(w-1, h-1, w-1, 1);
				g.drawLine(w-1, h-1, 1, h-1);
				break;
			case RAISED_THICK:
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawLine(0,0,w-1,0);
				g.drawLine(0,1,w-2,1);
				g.drawLine(0,0,0,h-1);
				g.drawLine(1,0,1,h-2);
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawLine(w-1, h-1, 1, h-1);
				g.drawLine(w-1, h-2, 2, h-2);
				g.drawLine(w-1, h-1, w-1, 1);
				g.drawLine(w-2, h-1, w-2, 2);
				break;
			case LOWERED_THICK:
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawLine(0,0,w-1,0);
				g.drawLine(0,1,w-2,1);
				g.drawLine(0,0,0,h-1);
				g.drawLine(1,0,1,h-2);
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawLine(w-1, h-1, 1, h-1);
				g.drawLine(w-1, h-2, 2, h-2);
				g.drawLine(w-1, h-1, w-1, 1);
				g.drawLine(w-2, h-1, w-2, 2);
				break;
			case ETCHED_OUT_THIN:
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawRect(0, 0, w-2, h-2);
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawRect(1, 1, w-2, h-2);
				break;
			case ETCHED_IN_THIN:
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawRect(0, 0, w-2, h-2);
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawRect(1, 1, w-2, h-2);
				break;
			case ETCHED_OUT_THICK:
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawRect(0, 0, w-4, h-4);
				g.drawRect(1, 1, w-4, h-4);
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawRect(2, 2, w-4, h-4);
				g.drawRect(3, 3, w-4, h-4);
				break;
			case ETCHED_IN_THICK:
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawRect(0, 0, w-4, h-4);
				g.drawRect(1, 1, w-4, h-4);
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawRect(2, 2, w-4, h-4);
				g.drawRect(3, 3, w-4, h-4);
				break;
			case QT_OUT:
				ThreeDUtils.drawQTBorder(g, w, h, Color.black, new Color(197,194,197), Color.white, new Color(131,129,131), new Color(164,161,164));
				break;
			case QT_IN:
				ThreeDUtils.drawQTBorder(g, w, h, Color.black, new Color(90,87,90), new Color(120,117,120), Color.white, new Color(197,194,197) );
				break;
			}
		
		g.translate(-x,-y);
	}

	public static final void drawBorder(Graphics g, int x, int y, int w, int h, Color left_top, Color right_bottom)  {
		g.setColor(left_top);
		g.drawLine(x,y,x,y+h-2);
		g.drawLine(x,y,x+w-2,y);
		
		g.setColor(right_bottom);
		g.drawLine(x+w-1, y+h-1, x+w-1, y+1);
		g.drawLine(x+w-1, y+h-1, x+1, y+h-1);
	}

	public static void drawDisabledState(Graphics g, JComponent c)  {
		Dimension size = c.getSize();
		
		g.setColor(ThreeDPrefs.disableColor);
		for (int y=0; y<size.height; ++y)
			{
			for (int x=(y % 3); x<size.width; x+=3)
				{
				g.drawLine(x,y,x,y);
				}
			}
	}

	static final void drawQTBorder(Graphics g, int w, int h, Color bounds, Color upperleft1, Color upperleft2, Color lowerright1, Color lowerright2)  {
		// auen
		g.setColor(bounds);
		g.drawLine(2,0,w-3,0);		g.drawLine(2,h-1,w-3,h-1);		g.drawLine(0,2,0,h-3);	g.drawLine(w-1,2,w-1,h-3);
		g.drawLine(1,1,1,1);			g.drawLine(w-2,1,w-2,1);		g.drawLine(1,h-2,1,h-2);	g.drawLine(w-2,h-2,w-2,h-2);
		// oben und links
		g.setColor(upperleft1);
		g.drawLine(2,1,w-3,1);		g.drawLine(1,2,1,h-3);
		g.setColor(upperleft2);
		g.drawLine(2,2,2,h-3);		g.drawLine(2,2,w-3,2);		g.drawLine(3,3,3,3);
		// unten und rechts
		g.setColor(lowerright1);
		g.drawLine(2,h-2,w-3,h-2);	g.drawLine(w-3,h-3,w-3,h-3);	g.drawLine(w-2,2,w-2,h-3);
		g.setColor(lowerright2);
		g.drawLine(3,h-3,w-4,h-3);	g.drawLine(w-4,h-4,w-4,h-4);	g.drawLine(w-3,3,w-3,h-4);
	}

	public static Insets getBorderInsets(int type)  {
		switch (type)
			{
			case LINE_LIGHT:
			case LINE_DARK:
			case RAISED_THIN:
			case LOWERED_THIN:
				return oneInsets;
			case RAISED_THICK:
			case LOWERED_THICK:
			case ETCHED_OUT_THIN:
			case ETCHED_IN_THIN:
				return twoInsets;
			case ETCHED_OUT_THICK:
			case ETCHED_IN_THICK:
				return fourInsets;
			case QT_OUT:
			case QT_IN:
				return fourInsets;
			}
		
		System.err.println("ThreeDUtils.getBorderInsets(): internal error");
		
		return fourInsets;
	}

	public static boolean isLoweredBorderType(int type)  {
		return ( (type == LOWERED_THIN) || (type == LOWERED_THICK) );
	}

}
