/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.threeD;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class ThreeD_InternalFrameTitlePane extends JPanel implements Border, MouseListener, ActionListener {
	JInternalFrame 	frame	 	= null;
	Font			font			= null;
	JButton			closeButton	= new JButton(new ThreeDIcons.InternalFrameIcon1());
	JButton			iconifyButton	= new JButton(new ThreeDIcons.InternalFrameIcon2());
	JButton			resizeButton	= new JButton(new ThreeDIcons.InternalFrameIcon3());
	JLabel			title			= new JLabel("") { 
										public Dimension getPreferredSize() { 
											return new Dimension(Short.MAX_VALUE, Math.max(closeButton.getSize().height, super.getPreferredSize().height)); 
										}};
										
	public ThreeD_InternalFrameTitlePane(JInternalFrame f)  {
		super();
		
		// Layout
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		setBorder(this);
		
		// Knpfe initialisieren
		closeButton.setOpaque(false);
		closeButton.setRequestFocusEnabled(false);
		closeButton.setMargin(new Insets(1,1,1,1));
		closeButton.setBorder(BorderFactory.createEmptyBorder(1,1,1,1));
		closeButton.addMouseListener(this);
		iconifyButton.setOpaque(false);
		iconifyButton.setRequestFocusEnabled(false);
		iconifyButton.setMargin(new Insets(1,1,1,1));
		iconifyButton.setBorder(BorderFactory.createEmptyBorder(1,1,1,1));
		iconifyButton.addMouseListener(this);
		resizeButton.setOpaque(false);
		resizeButton.setRequestFocusEnabled(false);
		resizeButton.setMargin(new Insets(1,1,1,1));
		resizeButton.setBorder(BorderFactory.createEmptyBorder(1,1,1,1));
		resizeButton.addMouseListener(this);
		title.setText(f.getTitle());
		title.setFont(ThreeDPrefs.internalFrameTitleFont);
		
		// Listeners
		closeButton.addActionListener(this);
		iconifyButton.addActionListener(this);
		resizeButton.addActionListener(this);
		
		// hinzufgen
		add(closeButton);
		add(Box.createHorizontalStrut(2));
		add(title);
		add(Box.createHorizontalStrut(2));
		add(iconifyButton);
		add(Box.createHorizontalStrut(2));
		add(resizeButton);
		
		frame = f;
		font = ThreeDPrefs.internalFrameTitleFont;
	}

	public void actionPerformed(java.awt.event.ActionEvent arg0)  {
		Object source = arg0.getSource();
		
		if (source == closeButton)
			{
			try {	frame.setClosed(true); } catch (PropertyVetoException e0) { }
			}
		else if (source == iconifyButton)
			{
			try { frame.setIcon( ! frame.isIcon()); } catch (PropertyVetoException e1) { }
			}
		else
			{
			try { frame.setMaximum(!frame.isMaximum()); } catch (PropertyVetoException e5) { }
			}
	}

	public java.awt.Insets getBorderInsets(java.awt.Component arg0)  {
		return new Insets(1,1,1,1);
	}

	public Dimension getMaximumSize()  {
		return new Dimension(Short.MAX_VALUE, getPreferredSize().height);
	}

	public Dimension getMinimumSize()  {
		return getPreferredSize();
	}

	public Dimension getPreferredSize()  {
		FontMetrics fm = getFontMetrics(font);
		
		int width = 200;
		int height = fm.getAscent() + fm.getDescent() + 4;
		
		return new Dimension(width, height);
	}

	public boolean isBorderOpaque()  {
		return true;
	}

	public void mouseClicked(java.awt.event.MouseEvent arg0)  {
	}

	public void mouseEntered(java.awt.event.MouseEvent arg0)  {
		JButton source = (JButton)arg0.getSource();
		source.setBorder(this);
	}

	public void mouseExited(java.awt.event.MouseEvent arg0)  {
		JButton source = (JButton)arg0.getSource();
		source.setBorder(BorderFactory.createEmptyBorder(1,1,1,1));
	}

	public void mousePressed(java.awt.event.MouseEvent arg0)  {
	}

	public void mouseReleased(java.awt.event.MouseEvent arg0)  {
	}

	public void paint(Graphics g)  {
		title.setText(frame.getTitle());
		
		remove(closeButton);
		remove(iconifyButton);
		remove(resizeButton);
		
		if (frame.isIconifiable()) add(iconifyButton, -1);
		if (frame.isResizable()) add(resizeButton, -1);
		if (frame.isClosable()) add(closeButton, 0);
		
		super.paint(g);
	}

	public void paintBorder(Component c, Graphics g, int x, int y, int w, int h)  {
		ThreeDUtils.drawBorder(ThreeDUtils.RAISED_THIN, g, x, y, w, h);
	}

	public void paintComponent(Graphics g)  {
		Dimension size = getSize();
		
		// Hintergrund malen
		g.setColor(frame.isSelected() ? ThreeDPrefs.highlightColor : ThreeDPrefs.backgroundColor);
		g.fillRect(0,0,size.width,size.height);
	}

}
